//------------------------------------------------------------------------
//
// Author      : Maarten "McCLaw" Kronberger
// Email       : sulacomcclaw@hotmail.com
// Website     : http://www.sulaco.co.za
// Date        : 1 April 2003
// Version     : 1.0
// Description : Skeletal Character animation using Keyframe interpolation and 
//               Milkshape 3D ASCII files
//
//------------------------------------------------------------------------
unit Animate;

interface

uses
  OpenGL,
  Model,
  Dialogs,
  timer;

type
  clsCamera = record
	  xangle, yangle, zangle : double;
  	dist : double;
  end;

type
  clsAnimate = class
  public
    AnimationFile : string;
    FramesPerSecond : double;
    Model : clsModel;
    angle_speed : double;
    dist_speed : double;
    show_bones : boolean;
    camera : clsCamera;
    mouse_x, mouse_y , mouse_b : integer;
    t : clsTimer;
	  time : single;
	  elapsedTime : single;

    { Condtructor }
    constructor create(iAnimationFile : string; iFramesPerSecond : double);

    { Render Te Model }
    procedure display(model : clsModel);

    { Render Model and Calculate FPS }
    procedure render();

    { Set OpenGL Projection }
    procedure set_projection();
  end;
implementation

{ clsAnimate }

{---------------------------------------------------------------------}
{  Condtructor                                                        }
{---------------------------------------------------------------------}
constructor clsAnimate.create(iAnimationFile: string; iFramesPerSecond: double);
begin

  camera.xangle := 10.0;
  camera.yangle := 90.0;
  camera.zangle := 0.0;
  camera.dist := 500.0;

  angle_speed := 0.5;
  dist_speed := 0.5;
  show_bones := true;
  AnimationFile := iAnimationFile;
  FramesPerSecond := iFramesPerSecond;

  model := clsModel.create;

	if( not model.loadFromMs3dAsciiFile( AnimationFile ) ) then
		ShowMessage( 'Couldn''t load the model' );

	glShadeModel (GL_SMOOTH);

	// Set the polygon mode to fill
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glPolygonMode (GL_BACK, GL_POINTS);

	glEnable (GL_DEPTH_TEST);

	glCullFace (GL_BACK);
	glEnable (GL_CULL_FACE);

	glEnable (GL_TEXTURE_2D);
	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	// Set the clear color to black
	glClearColor(0.0, 0.0, 0.0, 0.0);

  // disabled for now (makes the model too dark)
  //	set_lighting();

	set_projection();

  t := clsTimer.create;
end;

{---------------------------------------------------------------------}
{  Set OpenGL Projection                                              }
{---------------------------------------------------------------------}
procedure clsAnimate.set_projection;
begin
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	glFrustum (-1.0, 1.0, -1.0, 1.0, 1.0, 1500.0);  
end;

{---------------------------------------------------------------------}
{  Render Te Model                                                    }
{---------------------------------------------------------------------}
procedure clsAnimate.display(model: clsModel);
begin
  // Clear the RGB buffer and the depth buffer
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
 
   // Set the modelview matrix to be the identity matrix
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity();

	// Set camera position
  glTranslatef ( 0.0, 0.0, -camera.dist);
	glRotatef (camera.xangle, 1.0, 0.0, 0.0);
	glRotatef (camera.yangle, 0.0, 1.0, 0.0);
	glRotatef (camera.zangle, 0.0, 0.0, 1.0);

  // Translation and rotation of the object goes here

	model.render();				// render model

	if(show_bones) then
	begin
		glClear( GL_DEPTH_BUFFER_BIT);	// clear z buffer
		model.renderBones();			// render bones
	end;
end;

{---------------------------------------------------------------------}
{  Render Model and Calculate FPS                                     }
{---------------------------------------------------------------------}
procedure clsAnimate.render();
begin
  display(model);

  // advance the animation in accordance to
	// elapsed time since last call
	elapsedTime := t.getTime() - time;
	time := time + elapsedTime;
	model.advanceAnimation( elapsedTime * FramesPerSecond);

end;

end.
