unit Timer;

interface
uses
  windows,
  mmsystem;

type pLARGE_INTEGER = ^LARGE_INTEGER;
type
  ttimer = record		 							    // Create A Structure For The Timer Information
    frequency : int64;							  // Timer Frequency
    resolution : single;							// Timer Resolution
    mm_timer_start : Longword;				// Multimedia Timer Start Value
    mm_timer_elapsed : Longword	;			// Multimedia Timer Elapsed Time
    performance_timer : boolean;			// Using The Performance Timer?
    performance_timer_start : int64;	// Performance Timer Start Value
    performance_timer_elapsed : int64;// Performance Timer Elapsed Time
  end;

type
  clsTimer = class
  public
    timer : ttimer;
	  constructor create();
    function  getTime() : single;	// in seconds since Timer creation
  end;
implementation

{ clsTimer }

constructor clsTimer.create;
begin
	// Check To See If A Performance Counter Is Available
	// If One Is Available The Timer Frequency Will Be Updated
	if ( not QueryPerformanceFrequency(timer.frequency)) then
	begin
		// No Performace Counter Available
		timer.performance_timer	:= FALSE;				// Set Performance Timer To FALSE
		timer.mm_timer_start	:= timeGetTime();			// Use timeGetTime() To Get Current Time
		timer.resolution	:= 1.0/1000.0;				// Set Our Timer Resolution To .001f
		timer.frequency		:= 1000;					// Set Our Timer Frequency To 1000
		timer.mm_timer_elapsed	:= timer.mm_timer_start;			// Set The Elapsed Time To The Current Time
	end
 	else
	begin
		// Performance Counter Is Available, Use It Instead Of The Multimedia Timer
		// Get The Current Time And Store It In performance_timer_start
		QueryPerformanceCounter(timer.performance_timer_start);
		timer.performance_timer		:= TRUE;				// Set Performance Timer To TRUE
		// Calculate The Timer Resolution Using The Timer Frequency
		timer.resolution		:= ((1.0)/(timer.frequency));
		// Set The Elapsed Time To The Current Time
		timer.performance_timer_elapsed	:= timer.performance_timer_start;
	end;
end;

function clsTimer.getTime: single;
var time : int64;								// time Will Hold A 64 Bit Integer
begin
	if (timer.performance_timer) then						// Are We Using The Performance Timer?
	begin
		QueryPerformanceCounter(time);		// Grab The Current Performance Time
		// Return The Current Time Minus The Start Time Multiplied By The Resolution
		result := ( ( time - timer.performance_timer_start) * timer.resolution);
	end
	else
	begin
		// Return The Current Time Minus The Start Time Multiplied By The Resolution
		result := (( timeGetTime() - timer.mm_timer_start) * timer.resolution);
	end;
end;

end.
